
use warnings;
use strict;

package Mozilla::Persona::Validate::Table;
use base 'Mozilla::Persona::Validate';

use Log::Report    qw/persona/;
use Digest         ();

=chapter NAME
Mozilla::Persona::Validate::Table - validate the existence via a simple table

=chapter SYNOPSIS
  # add to the configation of $domain.json
  { ...
    "validate" : {
      "class"  : "Mozilla::Persona::Validator::Table",
      "pwfile" : "/etc/persona/passwords",
      "domain" : "example.com",
    },
    ...
  }

=chapter DESCRIPTION
The validator object must ensure that the user/password combination
is valid.  In this case, a simple password-file-like table is used
to authenticate the user.

You may put that file with the other configuration information
in F</etc/persona/>, for instance under the name C<passwords>.  This
is the default when you run C<persona setup>.

=section Password file syntax

Lines which are blank (visually empty) or start with a '#' (comment)
are ignored.  Other lines describe users.

The user definition lines contain (currently) three fields separated by a
colon ':'.  First a username, than the digest algorithm used (acceptable
to M<Digest>, you may need to install additional Digest modules) and
finally the base64 encoded version of that digest.  The username may
either include the domain, or the domain must be set in the configuration.

Example of two lines in the C<passwords> file:

  # my test user
  test@example.com:MD5:47b0eb54a2f09e1b97881eb8746486ce

With "domain" set to "example.com", your usernames become easier

  test:MD5:47b0eb54a2f09e1b97881eb8746486ce

You may still use full usernames if you want.  Full usernames which
are in an other domain are silently dropped.

B<Be warned> that md5sum produces hex, where we need base64 here.
Other scripts output base64 directly, for instance F<sha256sum>.  You
may use that to produce the third field of a user defining line:

  echo -n "my-password" | sha256sum
  echo "my-password\c"  | sha256sum

=chapter METHODS

=section Constructors

=c_method new OPTIONS
=requires pwfile FILENAME
The password file.  Probably, you want to provide an absolute path.
When the file changes, it will automatically be reloaded.

=option  domain DOMAIN
=default domain C<undef>
Default domain for all users in the file.
=cut

sub init($)
{   my ($self, $args) = @_;

    my $fn = $args->{pwfile} or panic;
    $self->openFile($fn);   # pre-load
    $self->{MPVT_domain} = $args->{domain};
    $self;
}

#------------
=section Accessors
=method pwfile
=method domain
=cut

sub pwfile() {shift->{MPVT_fn}}
sub domain() {shift->{MPVT_domain}}

sub openFile(;$)
{   my ($self, $fn) = @_;

    if($fn) { $self->{MPVT_fn} = $fn }
    else    { $fn = $self->{MPVT_fn} }

    my $mtime = (stat $fn)[9];
    defined $mtime
        or fault __x"passwd file {file}", file => $fn;

    if(my $last_mtime = $self->{MPVT_mtime})
    {   return $self->{MPVT_info} if $mtime eq $last_mtime;
    }

    my $domain = $self->domain;
    my %info;
    open PASSWD, '<:raw', $fn
        or fault __x"cannot read file {file}", file => $fn;

    while(<PASSWD>)
    {   next if m/^#|^\s*$/;
        chomp;
        my ($user, $algo, $passwd) = split /\:/;
        if(index $user, '@' < 0)
        {   # abbreviated usedname
            $user .= '@' . $domain;
        }
        elsif($user !~ m/\@\Q$domain\E$/i) 
        {   # account for other domain
            next;
        }

        defined $passwd
            or warning __x"not enough fields in {file} line {linenr}"
                 , file => $fn, linenr => $.;

        !$info{$user}
            or warning __x"found user {user} again in {file} line {linenr}"
                 , file => $fn, linenr => $.;

        $info{$user} = [$algo, $passwd];
    }
    close PASSWD
        or fault __x"read errors in {file}", file => $fn;

    $self->{MPVT_mtime} = $mtime;
    $self->{MPVT_info}  = \%info;
}

sub isValid($$)
{   my ($self, $user, $password) = @_;

    my $info = $self->openFile->{$user}
        or return 0;   # unknown user
   
    my ($algo, $expect) = @$info;
    my $digester = eval { Digest->new($algo) };
    error __x"unsupported digest algorithm {name} for {user}"
     , name => $algo, user => $user
       if $@;

    $expect eq $digester->add($password)->b64digest;
}

1;
